"""Module for a json formatted logger"""

import logging

import datetime as dt

from typing import Dict, Any

from pythonjsonlogger import jsonlogger


def get_logger(name: str) -> logging.Logger:
    """Returns logger instance"""
    logging.captureWarnings(True)
    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)
    if logger.hasHandlers():
        return logger
    handler = logging.StreamHandler()
    handler.setFormatter(
        logging.Formatter(
            "[%(asctime)s] %(threadName)s %(name)s [%(levelname)7s] %(message)s"
        )
    )
    logger.addHandler(handler)
    return logger


def get_logger_json(name: str = "logger") -> logging.Logger:
    """Function to get a new logger instance"""
    log = logging.getLogger(name)
    log.setLevel(logging.INFO)
    if not log.handlers:
        log.addHandler(get_app_logging_handler())
    return log


def get_app_logging_handler() -> logging.StreamHandler:
    """Getting a new logging handler"""
    log_handler = logging.StreamHandler()
    formatter = JsonLogFormatter()
    log_handler.setFormatter(formatter)
    return log_handler


class JsonLogFormatter(jsonlogger.JsonFormatter):
    """Json Log formatter following the Zooplus standards."""

    def add_fields(
        self,
        log_record: Dict[str, Any],
        record: logging.LogRecord,
        message_dict: Dict[str, str],
    ) -> None:
        """Adding json fields for logging"""
        super().add_fields(log_record, record, message_dict)

        log_record["log_type"] = "application_log"
        log_record["@timestamp"] = dt.datetime.now(dt.timezone.utc).isoformat()
        log_record["description"] = log_record.pop("message", None)
        log_record["severity"] = record.levelname.upper()
        log_record["class"] = ":".join(
            [record.module, record.funcName, str(record.lineno)]
        )

        exc_info = log_record.pop("exc_info", None)
        if exc_info:
            log_record["stacktrace"] = exc_info
        if record.process:
            log_record["pid"] = record.process
        if record.threadName:
            log_record["thread"] = record.threadName



"""Direct imports for package"""

from stefan_dataframe_utils.df_cache import df_cache
from stefan_dataframe_utils.df_optimize import df_optimize
from stefan_dataframe_utils.logger import get_logger, get_logger_json

__all__ = ["df_cache", "df_optimize", "logger"]



"""Caching decorator for functions returning pd.DataFrame objects"""
import datetime as dt
import os
from functools import wraps
from typing import Any

import pandas as pd

from .logger import get_logger

log = get_logger(__name__)


def df_cache(func):
    """Caching decorator for functions returning pd.DataFrame"""
    cache_dir = "cache/"

    @wraps(func)
    def inner_func(*args: Any, **kwargs: Any):
        """Function to be decorated"""
        file_name = cache_dir + func.__name__
        if not args and not kwargs:
            file_name += f"_run_{dt.date.today().isoformat()}.snappy.parquet"
        else:
            file_name = (
                file_name
                + "_"
                + "_".join(map(str, args))
                + "_"
                + "_".join([f"{str(k)}__{str(v)}" for k, v in kwargs.items()])
                + ".snappy.parquet"
            )
        if os.path.isfile(file_name):
            log.info("Reading %s from cache", func.__name__)
            return pd.read_parquet(file_name)

        result = func(*args, **kwargs)
        if not os.path.exists(cache_dir):
            os.makedirs(cache_dir)
        log.info("Caching %s", func.__name__)
        result.to_parquet(file_name)
        return result

    return inner_func
